Imports System.ComponentModel
Imports System.Windows.Forms
Namespace ValidatedControls
    Public MustInherit Class ValidatedTextBox
        Inherits TextBox
        Implements IValidatedControl


        Private m_allowedChars As String = ""
        Private m_allowedActions As List(Of System.Char)
        Private m_TextMinLength As Integer = 0
        Private m_TextMaxLength As Integer = 1000
        Private m_TextNullAllowed As Boolean = True
        Protected m_ControlDescription As String = ""

        Private m_ErrorMessage As String = ""
        'darf Punkte, Striche ... enthalten,
        'wird in der Text-Eigenschaft beim Read "gefiltert"
        Private m_ValidatedText As String = ""
        'Private m_IsValide As Boolean = False

        Private m_FilterChar As Char

        Private m_SuppressEvents As Boolean = False
        Private m_InternalEventSuppress As Boolean = False

        Private m_ErrorControlIndex As Integer = 0

        Private m_Status As TextBoxStaten = TextBoxStaten.Enabled

        'Public Event ErrorMessageChanged(ByVal Sender As Object, ByVal ErrorControlIndex As Integer, ByVal ErrorMessage As String) Implements IErrorProvidingControl.ErrorMessageChanged
        Public Event Changed(ByVal Sender As IValidatedControl, ByVal ErrorControlIndex As Integer, ByVal ErrorMessage As String, ByVal ValidatedValue As String) Implements IValidatedControl.Changed
        <Description("Gibt bei vorhandenem Fehler leeren String, ohne Fehler den gefilterten Text zurck."), Category("ValidatedTextBox")> _
        Public ReadOnly Property ValidatedAndFilteredValue() As String Implements IValidatedControl.ValidatedAndFilteredValue
            Get
                Return m_ValidatedText.Replace(m_FilterChar, "")
            End Get
        End Property

        <Description("Legt fest und gibt zurck, ob die TextBox Enabled, Disabled oder Inaktiv ist."), Category("ValidatedTextBox")> _
        Property Status() As TextBoxStaten
            Get
                Return m_Status
            End Get
            Set(ByVal value As TextBoxStaten)
                If m_Status <> value Then
                    m_InternalEventSuppress = True

                    m_Status = value
                    Select Case m_Status
                        Case TextBoxStaten.Disabled
                            Me.Enabled = False
                            m_SuppressEvents = False
                        Case TextBoxStaten.Enabled
                            Me.Enabled = True
                            m_SuppressEvents = False
                        Case TextBoxStaten.Inactive
                            Me.Text = ""
                            Me.Enabled = False
                            m_SuppressEvents = True
                    End Select
                    m_InternalEventSuppress = False

                End If
            End Set
        End Property

        <Description("Zeichen, das beim Lesen der Property Text gefiltert wird."), Category("ValidatedTextBox")> _
        Property FilterChar() As Char
            Get
                Return m_FilterChar
            End Get
            Set(ByVal value As Char)
                m_FilterChar = value
            End Set
        End Property
        <Description("Beim Lesen: ungefilterter Text, beim Schreiben: Text, den die TextBox anzeigen soll (wird vor Anzeige formatiert)."), Category("ValidatedTextBox")> _
        Shadows Property Text() As String
            Get
                Return MyBase.Text
            End Get
            Set(ByVal value As String)
                ValidateFormatAndRaiseEvents(value.Trim)
            End Set
        End Property

        <Description("Fehlermeldung."), Category("ValidatedTextBox")> _
        ReadOnly Property ErrorMessage() As String Implements IValidatedControl.ErrorMessage
            Get
                Return m_ErrorMessage
            End Get
        End Property

        <Description("In der TextBox erlaubte Zeichen."), Category("ValidatedTextBox")> _
        Property allowedChars() As String
            Get
                Return m_allowedChars
            End Get
            Set(ByVal value As String)
                m_allowedChars = value
            End Set
        End Property
        <Description("Minimale Lnge der Zeichenfolge in der TextBox."), Category("ValidatedTextBox")> _
        Property TextMinLength() As Integer
            Get
                Return m_TextMinLength
            End Get
            Set(ByVal value As Integer)
                m_TextMinLength = value
                If value = 0 Then
                    m_TextNullAllowed = True
                End If
            End Set
        End Property
        <Description("Maximale Lnge der Zeichenfolge in der TextBox."), Category("ValidatedTextBox")> _
        Property TextMaxLength() As Integer
            Get
                Return m_TextMaxLength
            End Get
            Set(ByVal value As Integer)
                m_TextMaxLength = value
            End Set
        End Property
        <Description("Legt fest und gibt zurck, ob leere Zeichenfolgen in der TextBox erlaubt sind."), Category("ValidatedTextBox")> _
        Property TextNullAllowed() As Boolean
            Get
                Return m_TextNullAllowed
            End Get
            Set(ByVal value As Boolean)
                m_TextNullAllowed = value
            End Set
        End Property
        <Description("Legt fest und gibt zurck, ob die TextBox Events ""ErrorMessageChanged"" und ""ValidatedValueChanged"" auslst."), Category("ValidatedTextBox")> _
        Public Property SuppressEvents() As Boolean Implements IValidatedControl.SuppressEvents
            Get
                Return m_SuppressEvents
            End Get
            Set(ByVal value As Boolean)
                m_SuppressEvents = value
            End Set
        End Property
        <Description("Legt fest welche Bezeichnung fr dieses Control in Fehlermeldungen verwendet  wird."), Category("ValidatedTextBox")> _
        Property ControlDescription() As String
            Get
                Return m_ControlDescription
            End Get
            Set(ByVal value As String)
                m_ControlDescription = value
            End Set
        End Property
        <Description("Index zur Zuordnung eines Fehlers fr eine Fehlerbearbeitung mit Eventhandlern."), Category("ValidatedTextBox")> _
        Public Property ErrorControlIndex() As Integer Implements IValidatedControl.ErrorControlIndex
            Get
                Return m_ErrorControlIndex
            End Get
            Set(ByVal value As Integer)
                m_ErrorControlIndex = value
            End Set
        End Property
        Public Sub New()
            m_allowedActions = New List(Of System.Char)

            m_allowedActions.Add(System.Convert.ToChar(Keys.Back))
            m_allowedActions.Add(System.Convert.ToChar(Keys.Enter))
            m_allowedActions.Add(System.Convert.ToChar(Keys.Tab))
            m_allowedActions.Add(System.Convert.ToChar(Keys.Cancel))
            m_allowedActions.Add(System.Convert.ToChar(Keys.Delete))
            m_allowedActions.Add(System.Convert.ToChar(Keys.Escape))

        End Sub

        'Beim Aufruf dieser Funktion ist schon geklrt, dass keine leeren Texte bergeben werden
        Friend MustOverride Function ExtraFormatAndValueCheck(ByVal StringToCheck As String) As String
        Friend MustOverride Function Formatieren(ByVal StringDaten As String) As String
        'Friend MustOverride Function Aktuelle_StringDaten_Liefern() As String

        Private Sub ValidatingTextBox_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Me.KeyPress
            If Not m_allowedActions.Contains(e.KeyChar) Then
                If Not m_allowedChars.Contains(e.KeyChar) Then
                    e.Handled = True
                Else
                    If MyBase.Text.Length >= m_TextMaxLength Then
                        If MyBase.SelectedText = "" Then
                            e.Handled = True
                        End If
                    End If

                End If
            End If
        End Sub

        Private Sub InactiveStatusRealisieren()
            Dim EventNotwendig As Boolean = False
            If m_ErrorMessage <> "" Then
                m_ErrorMessage = ""
                EventNotwendig = True
            End If
            If m_ValidatedText <> "" Then
                m_ValidatedText = ""
                EventNotwendig = True
            End If
            MyBase.Text = ""
            If EventNotwendig Then
                RaiseEvent Changed(Me, Me.m_ErrorControlIndex, "", "")
            End If
        End Sub

        Private Sub ValidatingTextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.TextChanged
            'tritt jedes Mal nach manueller oder durch das Programm verursachter Textnderung ein

            If m_InternalEventSuppress Then Exit Sub
            If Me.m_Status = TextBoxStaten.Inactive Then
                InactiveStatusRealisieren()
                Exit Sub
            End If


            Dim StringToCheck As String = Preformat(MyBase.Text.Trim)
            Dim EventNotwendig As Boolean = False

            Dim GesamtFehler As String = FormatAndValueCheck(StringToCheck)
            If GesamtFehler <> m_ErrorMessage Then
                EventNotwendig = True
            End If
            Dim OldValidatedText As String = m_ValidatedText
            If GesamtFehler <> "" Then
                m_ValidatedText = ""
                m_ErrorMessage = GesamtFehler
            Else
                m_ValidatedText = Formatieren(StringToCheck)
                m_ErrorMessage = ""
            End If

            If Not m_SuppressEvents Then
                If OldValidatedText <> m_ValidatedText Then
                    EventNotwendig = True
                End If
                If EventNotwendig Then
                    RaiseEvent Changed(Me, Me.m_ErrorControlIndex, m_ErrorMessage, ValidatedAndFilteredValue)
                End If
            End If

        End Sub

        Private Sub ValidatingTextBox_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.Validating
            ValidateFormatAndRaiseEvents(MyBase.Text.Trim)
        End Sub
        Private Sub ValidateFormatAndRaiseEvents(ByVal Value As String)

            If Me.m_Status = TextBoxStaten.Inactive Then
                InactiveStatusRealisieren()
                Exit Sub
            End If

            Dim TrimmedValue As String = Preformat(Value.Trim)
            m_InternalEventSuppress = True

            Dim EventNotwendig As Boolean = False

            Dim GesamtFehler As String = FormatAndValueCheck(TrimmedValue)
            If GesamtFehler <> m_ErrorMessage Then
                EventNotwendig = True
            End If
            Dim OldValidatedText As String = m_ValidatedText
            If GesamtFehler <> "" Then
                m_ValidatedText = ""
                Try
                    MyBase.Text = Formatieren(TrimmedValue)
                Catch ex As Exception
                    MyBase.Text = TrimmedValue
                End Try

                m_ErrorMessage = GesamtFehler
            Else
                m_ValidatedText = Formatieren(TrimmedValue)
                MyBase.Text = m_ValidatedText
                m_ErrorMessage = ""
            End If

            If Not m_SuppressEvents Then
                If OldValidatedText <> m_ValidatedText Then
                    EventNotwendig = True
                End If

                If EventNotwendig = True Then
                    RaiseEvent Changed(Me, Me.m_ErrorControlIndex, m_ErrorMessage, ValidatedAndFilteredValue)
                End If
            End If

            m_InternalEventSuppress = False
        End Sub

        Private Function FormatAndValueCheck(ByVal StringToCheck As String) As String
            'Sofortanzeige fr zu kurze, zu lange Zeichenfolge oder unzulassiges Zeichen


            Select Case StringToCheck.Length
                Case 0
                    'Fehler, wenn leerer Text nicht erlaubt
                    If m_TextNullAllowed Then
                        Return ""
                    Else
                        Return m_ControlDescription & " darf nicht leer sein."

                        'If m_TextMinLength = m_TextMaxLength Then
                        '    Return m_ControlDescription & " muss genau " & TextFunktionen.DutzendText(m_TextMinLength) & " Zeichen enthalten."
                        'Else
                        '    Return m_ControlDescription & " muss mindestens " & TextFunktionen.DutzendText(m_TextMinLength) & " Zeichen enthalten."
                        'End If
                    End If
                Case Is > m_TextMaxLength
                    If m_TextMinLength = m_TextMaxLength Then
                        Return m_ControlDescription & " enthlt zu viele Zeichen (genau " & TextFunktionen.DutzendText(m_TextMaxLength) & " sind erlaubt)."
                    Else
                        Return m_ControlDescription & " enthlt zu viele Zeichen (max. " & TextFunktionen.DutzendText(m_TextMaxLength) & " sind erlaubt)."
                    End If
                Case Is < m_TextMinLength
                    Dim NullErlaubtString As String = ""
                    If m_TextNullAllowed Then NullErlaubtString = "oder eine leere Zeichenfolge "
                    If m_TextMinLength = m_TextMaxLength Then
                        Return m_ControlDescription & " muss genau " & TextFunktionen.DutzendText(m_TextMinLength) & " Zeichen " & NullErlaubtString & "enthalten."
                    Else
                        Return m_ControlDescription & " muss mindestens " & TextFunktionen.DutzendText(m_TextMinLength) & " Zeichen " & NullErlaubtString & "enthalten."
                    End If
                Case Else
                    For I As Integer = 0 To StringToCheck.Length - 1
                        If Not m_allowedChars.Contains(StringToCheck.Chars(I)) Then
                            Return m_ControlDescription & " enthlt unzulssiges Zeichen: " & Chr(34) & StringToCheck.Chars(I) & Chr(34) & "."
                        End If
                    Next
            End Select

            'Anmerkung: An diese Funktion werden bereits keine leeren Strings mehr bergeben
            Return ExtraFormatAndValueCheck(StringToCheck)
        End Function
        Friend Overridable Function Preformat(ByVal Text As String) As String
            Return Text
        End Function




    End Class

End Namespace
